
#ifndef __IDE_BUSMASTER_H__
#define __IDE_BUSMASTER_H__

#define	IO_WD1	        0x1F0		/* Primary Fixed Disk Controller */
#define	IO_WD2	        0x170		/* Secondary Fixed Disk Controller */

#define IOMASK          0xfffffffc

#define ATA_BM_OFFSET1			0x08
#define ATA_DMA_ENTRIES			256
#define ATA_DMA_EOT			0x80000000
#define ATA_IOSIZE			0x08
#define ATA_ALTOFFSET			0x206	/* alternate registers offset */
#define ATA_PCCARD_ALTOFFSET		0x0e	/* do for PCCARD devices */
#define ATA_ALTIOSIZE			0x01	/* alternate registers size */
#define ATA_BMIOSIZE			0x20

#define DEV_BSIZE                       512

#define ATAPI_MAGIC_LSB			0x14
#define ATAPI_MAGIC_MSB			0xeb

#define PCI_DEV_HW_SPEC_BM(idhi, idlo) \
    { #idlo, 4, 0x##idlo, #idhi, 4, 0x##idhi}

typedef struct _BUSMASTER_CONTROLLER_INFORMATION {
    PCHAR   VendorId;
    ULONG   VendorIdLength;
    ULONG   nVendorId;
    PCHAR   DeviceId;
    ULONG   DeviceIdLength;
    ULONG   nDeviceId;
    CHAR    VendorIdStr[4];
    CHAR    DeviceIdStr[4];
    ULONG   slotNumber;
    CHAR    channel;
//    CHAR    numOfChannes;
    CHAR    MasterDev;
}BUSMASTER_CONTROLLER_INFORMATION, *PBUSMASTER_CONTROLLER_INFORMATION;

typedef struct _BUSMASTER_CTX {
    PBUSMASTER_CONTROLLER_INFORMATION* BMListPtr;
    ULONG* BMListLen;
} BUSMASTER_CTX, *PBUSMASTER_CTX;

BUSMASTER_CONTROLLER_INFORMATION const BusMasterAdapters[] = {
    PCI_DEV_HW_SPEC_BM( 1230, 8086 ), //Intel PIIX ATA controller
    PCI_DEV_HW_SPEC_BM( 7010, 8086 ), //Intel PIIX3 ATA controller
    PCI_DEV_HW_SPEC_BM( 7111, 8086 ), //Intel PIIX4 ATA33 controller
    PCI_DEV_HW_SPEC_BM( 7199, 8086 ), // -- // --
    PCI_DEV_HW_SPEC_BM( 2421, 8086 ), //Intel ICH0 ATA33 controller
    PCI_DEV_HW_SPEC_BM( 2411, 8086 ), //Intel ICH ATA66 controller
    PCI_DEV_HW_SPEC_BM( 244a, 8086 ), //Intel ICH2 ATA100 controller
    PCI_DEV_HW_SPEC_BM( 244b, 8086 ), // -- // --

    PCI_DEV_HW_SPEC_BM( 5229, 10b9 ), //AcerLabs Aladdin ATA/ATA33 controller

    PCI_DEV_HW_SPEC_BM( 0571, 1106 ), //VIA ATAxxx controller

    PCI_DEV_HW_SPEC_BM( 5513, 1039 ), //SiS 5591 ATA33 controller

    PCI_DEV_HW_SPEC_BM( 0649, 1095 ), //CMD 649 ATA100 controller
    PCI_DEV_HW_SPEC_BM( 0648, 1095 ), //CMD 648 ATA66 controller
    PCI_DEV_HW_SPEC_BM( 0646, 1095 ), //CMD 646 ATA controller

    PCI_DEV_HW_SPEC_BM( c693, 1080 ), //Cypress 82C693 ATA controller

    PCI_DEV_HW_SPEC_BM( 0102, 1078 ), //Cyrix 5530 ATA33 controller

    PCI_DEV_HW_SPEC_BM( 7409, 1022 ), //AMD 756 ATA66 controller

    PCI_DEV_HW_SPEC_BM( 0211, 1166 ), //ServerWorks ROSB4 ATA33 controller

    PCI_DEV_HW_SPEC_BM( 4d33, 105a ), //Promise ATA33 controller
    PCI_DEV_HW_SPEC_BM( 4d38, 105a ), //Promise ATA66 controller
    PCI_DEV_HW_SPEC_BM( 0d30, 105a ), //Promise ATA100 controller
    PCI_DEV_HW_SPEC_BM( 4d30, 105a ), // -- // --

    PCI_DEV_HW_SPEC_BM( 0004, 1103 )  //HighPoint ATAxxx controller
};

#define NUM_BUSMASTER_ADAPTERS (sizeof(BusMasterAdapters) / sizeof(BUSMASTER_CONTROLLER_INFORMATION))

#define PCI_DEV_CLASS_STORAGE        0x01
#define PCI_DEV_SUBCLASS_IDE         0x01

typedef struct _IDE_BUSMASTER_REGISTERS {
    UCHAR Command;
    UCHAR DeviceSpecific0;
    UCHAR Status;
    UCHAR DeviceSpecific1;
    ULONG PRD_Table;
} IDE_BUSMASTER_REGISTERS, *PIDE_BUSMASTER_REGISTERS;

#define BM_STATUS_ACTIVE       0x01
#define BM_STATUS_ERR          0x02
#define BM_STATUS_INTR         0x04
#define BM_STATUS_DRIVE_0_DMA  0x20
#define BM_STATUS_DRIVE_1_DMA  0x40
#define BM_STATUS_SIMPLEX_ONLY 0x80

#define IsBusMaster(pciData) \
    ( ((pciData)->Command & (PCI_ENABLE_BUS_MASTER/* | PCI_ENABLE_IO_SPACE*/)) == \
          (PCI_ENABLE_BUS_MASTER/* | PCI_ENABLE_IO_SPACE*/))

#define PCI_IDE_PROGIF_NATIVE_1    0x01
#define PCI_IDE_PROGIF_NATIVE_2    0x04
#define PCI_IDE_PROGIF_NATIVE_ALL  0x05

#define IsMasterDev(pciData) \
    ( ((pciData)->ProgIf & 0x80) && \
      ((pciData)->ProgIf & PCI_IDE_PROGIF_NATIVE_ALL) != PCI_IDE_PROGIF_NATIVE_ALL )

// Device extension
typedef struct _HW_DEVICE_EXTENSION {
    PSCSI_REQUEST_BLOCK CurrentSrb;    // Current request on controller.
    PIDE_REGISTERS_1 BaseIoAddress1[2];    // Base register locations
    PIDE_REGISTERS_2 BaseIoAddress2[2];
    ULONG InterruptLevel;    // Interrupt level
    ULONG InterruptMode;    // Interrupt Mode (Level or Edge)
    PUSHORT DataBuffer;    // Data buffer pointer.
    ULONG WordsLeft;    // Data words left.
    // Number of channels being supported by one instantiation
    // of the device extension. Normally (and correctly) one, but
    // with so many broken PCI IDE controllers being sold, we have
    // to support them.
    ULONG NumberChannels;
    ULONG ErrorCount;    // Count of errors. Used to turn off features.
    ULONG DiscsPresent[4];    // Indicates number of platters on changer-ish devices.
    USHORT DeviceFlags[4];    // Flags word for each possible device.
    // Indicates the number of blocks transferred per int. according to the
    // identify data.
    UCHAR MaximumBlockXfer[4];
    BOOLEAN ExpectingInterrupt;    // Indicates expecting an interrupt
    BOOLEAN RDP;    // Indicate last tape command was DSC Restrictive.
    BOOLEAN DriverMustPoll;    // Driver is being used by the crash dump utility or ntldr.
    BOOLEAN DWordIO;    // Indicates use of 32-bit PIO
    // Indicates whether '0x1f0' is the base address. Used
    // in SMART Ioctl calls.
    BOOLEAN PrimaryAddress;
    // Placeholder for the sub-command value of the last
    // SMART command.
    UCHAR SmartCommand;
    // Placeholder for status register after a GET_MEDIA_STATUS command
    UCHAR ReturningMediaStatus;
    UCHAR Reserved[1];
    IDENTIFY_DATA FullIdentifyData;    // Identify data for device
    IDENTIFY_DATA2 IdentifyData[4];
    ULONG NumOfSectors[4];
    PSCSI_REQUEST_BLOCK OriginalSrb;    // Mechanism Status Srb Data
    SCSI_REQUEST_BLOCK InternalSrb;
    MECHANICAL_STATUS_INFORMATION_HEADER MechStatusData;
    SENSE_DATA MechStatusSense;
    ULONG MechStatusRetryCount;
    // BusMaster specific data
    BOOLEAN BusMaster;
    PIDE_BUSMASTER_REGISTERS  BaseIoAddressBM_0;
    PIDE_BUSMASTER_REGISTERS  BaseIoAddressBM[2];
    ULONG DevID;
} HW_DEVICE_EXTENSION, *PHW_DEVICE_EXTENSION;

// Logical unit extension
typedef struct _HW_LU_EXTENSION {
   ULONG Reserved;
} HW_LU_EXTENSION, *PHW_LU_EXTENSION;

extern UCHAR         pciBuffer[256];
extern PBUSMASTER_CONTROLLER_INFORMATION BMList;
extern ULONG         BMListLen;

extern VOID
AtapiEnumBusMasterController(
    IN PVOID DriverObject,
    PVOID Argument2
    );

extern ULONG
AtapiFindBusMasterController(
    IN PVOID HwDeviceExtension,
    IN PVOID Context,
    IN PVOID BusInformation,
    IN PCHAR ArgumentString,
    IN OUT PPORT_CONFIGURATION_INFORMATION ConfigInfo,
    OUT PBOOLEAN Again
    );

#define ChangePciConfig1(offs, op) {                             \
    UCHAR a;                                                     \
    ScsiPortGetBusDataByOffset(HwDeviceExtension,                \
                               PCIConfiguration, \
                               ConfigInfo->SystemIoBusNumber,    \
                               slotNumber,                       \
                               &a,                               \
                               offs,                             \
                               1);                               \
    a = op;                                                      \
    ScsiPortSetBusDataByOffset(HwDeviceExtension,                \
                               PCIConfiguration, \
                               ConfigInfo->SystemIoBusNumber,    \
                               slotNumber,                       \
                               &a,                               \
                               offs,                             \
                               1);                               \
}

#define SetPciConfig2(offs, op) {                                \
    USHORT a = op;                                               \
    ScsiPortSetBusDataByOffset(HwDeviceExtension,                \
                               PCIConfiguration, \
                               ConfigInfo->SystemIoBusNumber,    \
                               slotNumber,                       \
                               &a,                               \
                               offs,                             \
                               2);                               \
}

#define ChangePciConfig4(offs, op) {                             \
    ULONG a;                                                     \
    ScsiPortGetBusDataByOffset(HwDeviceExtension,                \
                               PCIConfiguration, \
                               ConfigInfo->SystemIoBusNumber,    \
                               slotNumber,                       \
                               &a,                               \
                               offs,                             \
                               4);                               \
    a = op;                                                      \
    ScsiPortSetBusDataByOffset(HwDeviceExtension,                \
                               PCIConfiguration, \
                               ConfigInfo->SystemIoBusNumber,    \
                               slotNumber,                       \
                               (PUCHAR)&a,                       \
                               offs,                             \
                               4);                               \
}

#define BrutePoint() { \
    __asm int 3        \
}

#endif //__IDE_BUSMASTER_H__